return {
	{
		"nvim-telescope/telescope.nvim",
		version = false,
		dependencies = {
			"nvim-lua/plenary.nvim",
			{ "nvim-telescope/telescope-fzf-native.nvim", build = "make" },
		},
		config = function()
			local telescope = require("telescope")
			telescope.setup({
				extension = {
					fzf = {},
				},
			})

			telescope.load_extension("fzf")
		end,
	},

	{
		"nvim-neo-tree/neo-tree.nvim",
		branch = "v3.x",
		dependencies = {
			"nvim-lua/plenary.nvim",
			"nvim-tree/nvim-web-devicons",
			"MunifTanjim/nui.nvim",
		},
		lazy = false,
		init = function()
			-- opens neotree focused and filling the entire window if no argument is passed
			if vim.fn.argc() == 0 then
				require("neo-tree").setup()
				-- position=current is the key here to open neotree on the entire screen and
				-- its the same thing as using filesystem.hijack_netrw_behavior = "open_current"
				vim.cmd("Neotree focus position=current")
			end
		end,
		opts = {},
	},

	{
		"folke/trouble.nvim",
		dependencies = { "nvim-tree/nvim-web-devicons" },
		opts = {},
	},

	{
		"folke/which-key.nvim",
		event = "VeryLazy",
		opts = {},
		init = function()
			vim.o.timeout = true
			vim.o.timeoutlen = 500
		end,
	},

	{
		"echasnovski/mini.map",
		version = false,
		init = function()
			local mini = require("mini.map")
			mini.setup({
				integrations = {
					mini.gen_integration.builtin_search(),
					mini.gen_integration.diagnostic(),
				},
				window = {
					focusable = true,
					width = 20,
					winblend = 0,
				},
			})
		end,
	},

	{
		"hedyhli/outline.nvim",
		lazy = true,
		cmd = { "Outline", "OutlineOpen" },
		keys = {
			{ "<leader>oo", "<cmd>Outline<CR>", desc = "[O]utline [O]pen" },
			{ "<leader>of", "<cmd>OutlineFocus<CR>", desc = "[O]utline [F]ocus" },
		},
		opts = {
			outline_window = {
				width = 20,
			},
		},
	},
}
