return {
	{
		"neovim/nvim-lspconfig",
		dependencies = {
			"saghen/blink.cmp",
			{ "j-hui/fidget.nvim", opts = {} },
		},
		config = function()
			local lspconfig = require("lspconfig")
			local capabilities = require("blink.cmp").get_lsp_capabilities()

			-- Markdown
			lspconfig.marksman.setup({
				capabilities = capabilities,
			})

			-- Python
			lspconfig.pyright.setup({
				capabilities = capabilities,
			})

			-- Rust
			lspconfig.rust_analyzer.setup({
				capabilities = capabilities,
				settings = {
					["rust-analyzer"] = {
						cargo = {
							buildScripts = {
								enable = true,
							},
						},
						procMacro = {
							enable = true,
						},
						checkOnSave = {
							command = "clippy",
						},
						hover = {
							actions = {
								references = { enable = true },
							},
						},
						inlayHints = {
							closureReturnTypeHints = { enable = "always" },
							lifetimeElisionHints = { enable = "always" },
							reborrowHints = { enable = "always" },
						},
					},
				},
			})

			lspconfig.taplo.setup({
				capabilities = capabilities,
			})

			-- Lua
			local lua_rtp = vim.split(package.path, ";")
			table.insert(lua_rtp, "lua/?.lua")
			table.insert(lua_rtp, "lua/?/init.lua")

			lspconfig.lua_ls.setup({
				capabilities = capabilities,
				settings = {
					Lua = {
						runtime = {
							version = "LuaJIT",
							path = lua_rtp,
						},
						workspace = {
							library = vim.api.nvim_get_runtime_file("", true),
						},
						diagnostics = {
							globals = { "vim" },
						},
						telemetry = {
							enable = false,
						},
					},
				},
			})

			-- Zig
			lspconfig.zls.setup({
				capabilities = capabilities,
			})

			-- Nim
			lspconfig.nim_langserver.setup({
				capabilities = capabilities,
			})

			-- Bash
			lspconfig.bashls.setup({
				capabilities = capabilities,
			})

			-- HTML
			lspconfig.html.setup({
				capabilities = capabilities,
				filetypes = { "html", "templ", "htmldjango", "handlebars" },
			})

			-- HTMX
			lspconfig.htmx.setup({
				capabilities = capabilities,
				filetypes = { "html", "templ", "htmldjango" },
			})

			-- CSS
			lspconfig.cssls.setup({
				capabilities = capabilities,
			})

			-- JSON
			lspconfig.jsonls.setup({
				capabilities = capabilities,
			})

			vim.api.nvim_create_autocmd("LspAttach", {
				group = vim.api.nvim_create_augroup("nvwynd-lsp-attach", { clear = true }),
				callback = function(event)
					-- does word highlighting (across the entire file) whenever the cursor is on one
					local client = vim.lsp.get_client_by_id(event.data.client_id)
					if client and client.server_capabilities.documentHighlightProvider then
						vim.api.nvim_create_autocmd({ "CursorHold", "CursorHoldI" }, {
							buffer = event.buf,
							callback = vim.lsp.buf.document_highlight,
						})

						vim.api.nvim_create_autocmd({ "CursorMoved", "CursorMovedI" }, {
							buffer = event.buf,
							callback = vim.lsp.buf.clear_references,
						})
					end
				end,
			})
		end,
	},

	{
		"folke/lazydev.nvim",
		ft = "lua",
		opts = {
			library = {
				{
					path = "luvit-meta/library",
					words = { "vim%.uv" },
				},
			},
		},
	},

	{
		"Bilal2453/luvit-meta",
		lazy = true,
	},
}
